/*!
 * @file baseRTC.cpp
 * @brief XML SERialization/DESerialization implementation file
 * @author SAGAMI, Tsuyoshi <sagami@brains.co.jp>
 */
#include <sys/types.h>
#include <sys/time.h>

#include <exception>
#include <iostream>
#include <sstream>

#include "baseRTC.h"

#include "xml_serdes.h"

using namespace std;
using namespace boost::serialization;
using namespace boost::archive;
using namespace XmlSerDes;

/* ================================================================ *
 *
 *  Boost::Serialization の仕様により，non-intrusiveなシリアライズ関数
 *  は下記のnamespace内部に置く必要があり，かつ，これを使用する関数
 *  からtemplate定義が可視でなくてはならない。
 *
 * ================================================================ */
namespace boost {
namespace serialization {

#define NVP(obj, member)  make_nvp(#member, obj.member)

/*! @brief @a baseRTC::Header用シリアライズ関数.
 *  @tparam Archive Boostのアーカイブクラス名
 *  @param[out] ar Archive Boostのアーカイブクラスのインスタンス
 *  @param[in] h シリアライズ／デシリアライズされる@a baseRTC::Header
 *  @param[in] (未使用) バージョン
 */
template <class Archive>
void
serialize(Archive& ar, baseRTC::Header& h, const unsigned int /* version */)
{
	ar & NVP(h, category);
	ar & NVP(h, type);
	ar & NVP(h, sender_name);
	ar & NVP(h, id);
	ar & NVP(h, tag);
	ar & NVP(h, time);
	ar & NVP(h, hdr_version);
}
#undef NVP

} /* namespace boost*/
} /* namespace serialization */

/* ================================================================ *
 *	file-local utilities in unnamed namespace
 * ================================================================ */
namespace /* unnamed */{
double
now()
{
	struct timeval tv;
	struct timezone tz;
	(void)gettimeofday(&tv, &tz);
	
	return double(tv.tv_sec) + 1.0e-6*tv.tv_usec;
}

} /* unnamed namespace */


/* ================================================================ *
 *	baseRTC namespace 
 * ================================================================ */
namespace baseRTC {

/* ---------------------------------------------------------------- *
 *	Header 
 * ---------------------------------------------------------------- */
const char Header::HEADER_TAG[] = "XmlSerDesHeader";

Header::Header()
	: category(),
	  type(),
	  sender_name(),
	  id(),
	  tag(),
	  time(now()),
	  hdr_version(HEADER_VERSION)
{
}
	
Header::Header(const std::string& category, const std::string& type, const std::string& sender_name, const std::string& id, const std::string& tag)
	: category(category),
	  type(type),
	  sender_name(sender_name),
	  id(id),
	  tag(tag),
	  time(now()),
	  hdr_version(HEADER_VERSION)
{
}
	
int
Header::readFromXml(istream& is)
{
	int ret = readFromXmlSub(is, *this, HEADER_TAG);
	if (ret < 0) {
		*this = Header();
	}
	return ret;
}

int
Header::readFromXml(const string& xml)
{
	istringstream is(xml);
	return readFromXmlSub(is, *this, HEADER_TAG);
}

int
Header::writeToXml(ostream& os) const
{
	return writeToXmlSub(os, *this, HEADER_TAG);
}

} // namespace baseRTC

/*
 * Local Variables:
 * mode: c++
 * c-basic-offset: 4
 * tab-width: 4
 * indent-tabs-mode: t
 * End:
 *
 */
